package com.cultural.heritage.controller.book;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cultural.heritage.annotation.AuthCheck;
import com.cultural.heritage.common.BaseResponse;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.common.ResultUtils;
import com.cultural.heritage.constant.UserConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.model.dto.CommonBatchRequest;
import com.cultural.heritage.model.dto.CommonRequest;
import com.cultural.heritage.model.dto.photoProducts.PhotoProductsAddRequest;
import com.cultural.heritage.model.dto.photoProducts.PhotoProductsQueryRequest;
import com.cultural.heritage.model.dto.photoProducts.PhotoProductsUpdateRequest;
import com.cultural.heritage.model.entity.PhotoProducts;
import com.cultural.heritage.model.vo.clothesinfo.PhotoProductsVO;
import com.cultural.heritage.service.book.PhotoCategoryService;
import com.cultural.heritage.service.book.PhotoProductsService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@RestController
@RequestMapping("/photoProducts")
@Slf4j
@Tag(name = "写真产品管理模块")
public class PhotoProductsController {
    

    @Resource
    private PhotoProductsService photoProductsService;


    @Resource
    private PhotoCategoryService photoCategoryService;


    /**
     * Web端管理员添加写真产品
     * @param photoProductsAddRequest 写真产品添加请求体
     * @return  写真产品id
     */
    @PostMapping("/add")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    @Operation(summary = "Web端管理员添加写真产品", description = "参数：写真产品添加请求体，权限：管理员(admin, boss)，方法名：addPhotoProducts")
    public BaseResponse<Long> addPhotoProducts(@RequestBody PhotoProductsAddRequest photoProductsAddRequest) {
        if (photoProductsAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        PhotoProducts photoProducts = new PhotoProducts();
        BeanUtils.copyProperties(photoProductsAddRequest, photoProducts);
        // 校验
        photoProductsService.validPhotoProducts(photoProducts, false);
        boolean result = photoProductsService.save(photoProducts);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "写真产品添加失败");
        return ResultUtils.success(photoProducts.getId());
    }




    /**
     * Web管理员更新写真产品
     * @param photoProductsUpdateRequest  写真产品更新请求体
     * @return 是否更新成功
     */
    @PostMapping("/update")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    @Operation(summary = "Web管理员更新写真产品", description = "参数：服装更新请求体，权限：管理员(admin, boss)，方法名：updatePhotoProducts")
    public BaseResponse<Boolean> updatePhotoProducts(@RequestBody PhotoProductsUpdateRequest photoProductsUpdateRequest) {
        if (photoProductsUpdateRequest == null || photoProductsUpdateRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        PhotoProducts photoProducts = new PhotoProducts();
        BeanUtils.copyProperties(photoProductsUpdateRequest, photoProducts);
        // 校验
        photoProductsService.validPhotoProducts(photoProducts, true);
        boolean result = photoProductsService.updateById(photoProducts);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "写真产品更新失败");
        return ResultUtils.success(true);
    }




    /**
     * Web管理员删除写真产品
     * @param commonRequest  写真产品id
     * @return  是否删除成功
     */
    @PostMapping("/delete")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    @Operation(summary = "Web管理员删除写真产品", description = "参数：写真产品id，权限：管理员(admin, boss)，方法名：deletePhotoProducts")
    public BaseResponse<Boolean> deletePhotoProducts(@RequestBody CommonRequest commonRequest) {
        if (commonRequest == null || commonRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = commonRequest.getId();
        boolean result = photoProductsService.removeById(id);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "写真产品删除失败");
        return ResultUtils.success(true);
    }




    /**
     * Web管理员批量删除写真产品
     * @param commonBatchRequest  写真产品id列表
     * @return  是否删除成功
     */
    @PostMapping("/delBatch")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    @Operation(summary = "Web管理员批量删除写真产品", description = "参数：写真产品id列表，权限：管理员(admin, boss)，方法名：deletePhotoProducts")
    public BaseResponse<Boolean> delBatchPhotoProducts(@RequestBody CommonBatchRequest commonBatchRequest) {
        if (commonBatchRequest == null || CollectionUtils.isEmpty(commonBatchRequest.getIdList())) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        List<Long> idList = commonBatchRequest.getIdList();
        boolean result = photoProductsService.removeBatchByIds(idList);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "写真产品批量删除失败");
        return ResultUtils.success(true);
    }




    /**
     * Web管理员分页查询写真产品
     * @param photoProductsQueryRequest   写真产品查询请求体
     * @return  写真产品列表
     */
    @PostMapping("/list/page")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    @Operation(summary = "Web管理员分页查询写真产品", description = "参数：写真产品查询请求体，权限：管理员(admin, boss)，方法名：listPhotoProductsByPage")
    public BaseResponse<Page<PhotoProductsVO>> listPhotoProductsByPage(@RequestBody PhotoProductsQueryRequest photoProductsQueryRequest) {
        if (photoProductsQueryRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        long current = photoProductsQueryRequest.getCurrent();
        long pageSize = photoProductsQueryRequest.getPageSize();
        QueryWrapper<PhotoProducts> queryWrapper = photoProductsService.getQueryWrapper(photoProductsQueryRequest);
        Page<PhotoProducts> page = photoProductsService.page(new Page<>(current, pageSize), queryWrapper);
        // 封装服装类VO
        List<PhotoProducts> records = page.getRecords();
        List<PhotoProductsVO> photoProductsVOS = records.stream().map(photoProducts -> {
            PhotoProductsVO photoProductsVO = new PhotoProductsVO();
            BeanUtils.copyProperties(photoProducts, photoProductsVO);
            return photoProductsVO;
        }).toList();
        Page<PhotoProductsVO> voPage = new Page<>();
        voPage.setRecords(photoProductsVOS);
        voPage.setTotal(page.getTotal());
        voPage.setCurrent(page.getCurrent());
        voPage.setSize(page.getSize());
        voPage.setPages(page.getPages());
        return ResultUtils.success(voPage);
    }




    /**
     * 小程序端用户根据id查询写真产品
     * @param commonRequest 写真产品id
     * @return  写真产品
     */
    @PostMapping("/list/detail")
    @Operation(summary = "小程序端用户根据id查询写真产品", description = "参数：写真产品id，权限：所有人，方法名：getPhotoProductsById")
    public BaseResponse<PhotoProductsVO> getPhotoProductsById(@RequestBody CommonRequest commonRequest) {
        if (commonRequest == null || commonRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = commonRequest.getId();
        PhotoProducts photoProducts = photoProductsService.getById(id);
        ThrowUtils.throwIf(photoProducts == null || photoProducts.getIsShelves() == 0, ErrorCode.OPERATION_ERROR, "该商品已被删除或者已下架");
        PhotoProductsVO photoProductsVO = new PhotoProductsVO();
        BeanUtils.copyProperties(photoProductsVO, photoProductsVO);
        return ResultUtils.success(photoProductsVO);
    }





    /**
     * Web端管理员上(下)架写真预约产品
     * @param commonRequest 写真产品id
     * @return 是否更新成功
     */
    @PostMapping("/shelves")
    @Operation(summary = "Web端管理员上(下)架写真预约产品", description = "参数：写真产品id，权限：管理员(admin, boss)，方法名：updatePhotoProductsShelvesStatus")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> updatePhotoProductsShelvesStatus(@RequestBody CommonRequest commonRequest) {
        if (commonRequest == null || commonRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        // 获取当前服务类商品的上(下)架状态
        Long id = commonRequest.getId();
        PhotoProducts photoProducts = photoProductsService.getById(id);
        Integer status = photoProducts.getIsShelves() == 0 ? 1 : 0;
        UpdateWrapper<PhotoProducts> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id", id);
        updateWrapper.set("isShelves", status);
        boolean update = photoProductsService.update(updateWrapper);
        ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "上架状态更新失败");
        return ResultUtils.success(true);
    }



}
